@@ -25,6 +25,8 @@ def get_sampler_dim(sampler_type):
         sampler_dim = 3
     elif sampler_type == "ExternalOES":
         sampler_dim = 2
+    elif sampler_type == "Buffer":
+        sampler_dim = 1
     else:
         assert False ("coord_dim: invalid sampler_type: " + sampler_type)
     return sampler_dim
@@ -40,6 +42,8 @@ def get_coord_dim(sampler_type):
 # Get the number of extra vector components (i.e. shadow comparitor)
 def get_extra_dim(sampler_type, use_proj, unused_fields):
     extra_dim = unused_fields
+    if sampler_type == "CubeArrayShadow":
+        return 0
     if sampler_type.find("Shadow") != -1:
         extra_dim += 1
     if use_proj:
@@ -47,6 +51,8 @@ def get_extra_dim(sampler_type, use_proj, unused_fields):
     return extra_dim
 
 def get_txs_dim(sampler_type):
+    if sampler_type.startswith("CubeArray"):
+        return 3
     if sampler_type.startswith("Cube"):
         return 2
     return get_coord_dim(sampler_type)
@@ -54,7 +60,7 @@ def get_txs_dim(sampler_type):
 def generate_sigs(g, tex_inst, sampler_type, variant = 0, unused_fields = 0):
     coord_dim = get_coord_dim(sampler_type)
     extra_dim = get_extra_dim(sampler_type, variant & Proj, unused_fields)
-    offset_dim = get_sampler_dim(sampler_type)
+    sampler_dim = get_sampler_dim(sampler_type)
 
     if variant & Single:
         return_type = "float"
@@ -71,15 +77,17 @@ def generate_sigs(g, tex_inst, sampler_type, variant = 0, unused_fields = 0):
         print "\n       (declare (in) " + vec_type("i" if tex_inst == "txf" else "", coord_dim + extra_dim) + " P)",
     if tex_inst == "txl":
         print "\n       (declare (in) float lod)",
-    elif tex_inst == "txf" or tex_inst == "txs":
+    elif ((tex_inst == "txf" or tex_inst == "txs") and "Buffer" not in sampler_type and "Rect" not in sampler_type):
         print "\n       (declare (in) int lod)",
     elif tex_inst == "txd":
-        grad_type = vec_type("", coord_dim)
+        grad_type = vec_type("", sampler_dim)
         print "\n       (declare (in) " + grad_type + " dPdx)",
         print "\n       (declare (in) " + grad_type + " dPdy)",
+    if sampler_type == "CubeArrayShadow" and tex_inst == "tex":
+        print "\n       (declare (in) float compare)",
 
     if variant & Offset:
-        print "\n       (declare (const_in) " + vec_type("i", offset_dim) + " offset)",
+        print "\n       (declare (const_in) " + vec_type("i", sampler_dim) + " offset)",
     if tex_inst == "txb":
         print "\n       (declare (in) float bias)",
 
@@ -105,7 +113,9 @@ def generate_sigs(g, tex_inst, sampler_type, variant = 0, unused_fields = 0):
             print "1",
 
         # Shadow comparitor
-        if sampler_type == "2DArrayShadow" or sampler_type == "CubeShadow": # a special case:
+        if sampler_type == "CubeArrayShadow": # a special case
+            print "(var_ref compare)",
+        elif sampler_type == "2DArrayShadow" or sampler_type == "CubeShadow": # a special case:
             print "(swiz w (var_ref P))",   # ...array layer is z; shadow is w
         elif sampler_type.endswith("Shadow"):
             print "(swiz z (var_ref P))",
@@ -115,7 +125,12 @@ def generate_sigs(g, tex_inst, sampler_type, variant = 0, unused_fields = 0):
     # Bias/explicit LOD/gradient:
     if tex_inst == "txb":
         print "(var_ref bias)",
-    elif tex_inst == "txl" or tex_inst == "txf" or tex_inst == "txs":
+    elif tex_inst == "txs" or tex_inst == "txf":
+        if "Rect" not in sampler_type and "Buffer" not in sampler_type:
+            print "(var_ref lod)",
+        else:
+            print "(constant int (0))"
+    elif tex_inst == "txl":
         print "(var_ref lod)",
     elif tex_inst == "txd":
         print "((var_ref dPdx) (var_ref dPdy))",
@@ -153,6 +168,11 @@ def generate_texture_functions(fs):
     generate_sigs("", "txs", "CubeShadow")
     generate_sigs("", "txs", "1DArrayShadow")
     generate_sigs("", "txs", "2DArrayShadow")
+    generate_fiu_sigs("txs", "2DRect")
+    generate_sigs("", "txs", "2DRectShadow")
+    generate_fiu_sigs("txs", "Buffer")
+    generate_fiu_sigs("txs", "CubeArray")
+    generate_sigs("", "txs", "CubeArrayShadow")
     end_function(fs, "textureSize")
 
     start_function("texture")
@@ -167,6 +187,11 @@ def generate_texture_functions(fs):
     generate_sigs("", "tex", "CubeShadow", Single);
     generate_sigs("", "tex", "1DArrayShadow", Single);
     generate_sigs("", "tex", "2DArrayShadow", Single);
+    generate_fiu_sigs("tex", "2DRect")
+    generate_sigs("", "tex", "2DRectShadow", Single);
+    # ARB_texture_cube_map_array extension
+    generate_fiu_sigs("tex", "CubeArray")
+    generate_sigs("", "tex", "CubeArrayShadow", Single);
 
     generate_fiu_sigs("txb", "1D")
     generate_fiu_sigs("txb", "2D")
@@ -174,6 +199,7 @@ def generate_texture_functions(fs):
     generate_fiu_sigs("txb", "Cube")
     generate_fiu_sigs("txb", "1DArray")
     generate_fiu_sigs("txb", "2DArray")
+    generate_fiu_sigs("txb", "CubeArray")
     generate_sigs("", "txb", "1DShadow", Single, 1);
     generate_sigs("", "txb", "2DShadow", Single);
     generate_sigs("", "txb", "CubeShadow", Single);
@@ -189,6 +215,9 @@ def generate_texture_functions(fs):
     generate_fiu_sigs("tex", "3D", Proj)
     generate_sigs("", "tex", "1DShadow", Proj | Single, 1);
     generate_sigs("", "tex", "2DShadow", Proj | Single);
+    generate_fiu_sigs("tex", "2DRect", Proj)
+    generate_fiu_sigs("tex", "2DRect", Proj, 1)
+    generate_sigs("", "tex", "2DRectShadow", Proj | Single);
 
     generate_fiu_sigs("txb", "1D", Proj)
     generate_fiu_sigs("txb", "1D", Proj, 2)
@@ -209,6 +238,8 @@ def generate_texture_functions(fs):
     generate_sigs("", "txl", "1DShadow", Single, 1);
     generate_sigs("", "txl", "2DShadow", Single);
     generate_sigs("", "txl", "1DArrayShadow", Single);
+    # ARB_texture_cube_map_array extension
+    generate_fiu_sigs("txl", "CubeArray")
     end_function(fs, "textureLod")
 
     start_function("textureLodOffset")
@@ -226,6 +257,8 @@ def generate_texture_functions(fs):
     generate_fiu_sigs("tex", "1D", Offset)
     generate_fiu_sigs("tex", "2D", Offset)
     generate_fiu_sigs("tex", "3D", Offset)
+    generate_fiu_sigs("tex", "2DRect", Offset)
+    generate_sigs("", "tex", "2DRectShadow", Offset | Single);
     generate_fiu_sigs("tex", "1DArray", Offset)
     generate_fiu_sigs("tex", "2DArray", Offset)
     generate_sigs("", "tex", "1DShadow", Offset | Single, 1);
@@ -246,14 +279,17 @@ def generate_texture_functions(fs):
     generate_fiu_sigs("txf", "1D")
     generate_fiu_sigs("txf", "2D")
     generate_fiu_sigs("txf", "3D")
+    generate_fiu_sigs("txf", "2DRect")
     generate_fiu_sigs("txf", "1DArray")
     generate_fiu_sigs("txf", "2DArray")
+    generate_fiu_sigs("txf", "Buffer")
     end_function(fs, "texelFetch")
 
     start_function("texelFetchOffset")
     generate_fiu_sigs("txf", "1D", Offset)
     generate_fiu_sigs("txf", "2D", Offset)
     generate_fiu_sigs("txf", "3D", Offset)
+    generate_fiu_sigs("txf", "2DRect", Offset)
     generate_fiu_sigs("txf", "1DArray", Offset)
     generate_fiu_sigs("txf", "2DArray", Offset)
     end_function(fs, "texelFetchOffset")
@@ -264,6 +300,9 @@ def generate_texture_functions(fs):
     generate_fiu_sigs("tex", "2D", Proj | Offset)
     generate_fiu_sigs("tex", "2D", Proj | Offset, 1)
     generate_fiu_sigs("tex", "3D", Proj | Offset)
+    generate_fiu_sigs("tex", "2DRect", Proj | Offset)
+    generate_fiu_sigs("tex", "2DRect", Proj | Offset, 1)
+    generate_sigs("", "tex", "2DRectShadow", Proj | Offset | Single);
     generate_sigs("", "tex", "1DShadow", Proj | Offset | Single, 1);
     generate_sigs("", "tex", "2DShadow", Proj | Offset | Single);
 
@@ -303,17 +342,23 @@ def generate_texture_functions(fs):
     generate_fiu_sigs("txd", "Cube")
     generate_fiu_sigs("txd", "1DArray")
     generate_fiu_sigs("txd", "2DArray")
+    generate_fiu_sigs("txd", "2DRect")
+    generate_sigs("", "txd", "2DRectShadow", Single);
     generate_sigs("", "txd", "1DShadow", Single, 1);
     generate_sigs("", "txd", "2DShadow", Single);
     generate_sigs("", "txd", "CubeShadow", Single);
     generate_sigs("", "txd", "1DArrayShadow", Single);
     generate_sigs("", "txd", "2DArrayShadow", Single);
+    # ARB_texture_cube_map_array extension
+    generate_fiu_sigs("txd", "CubeArray")
     end_function(fs, "textureGrad")
 
     start_function("textureGradOffset")
     generate_fiu_sigs("txd", "1D", Offset)
     generate_fiu_sigs("txd", "2D", Offset)
     generate_fiu_sigs("txd", "3D", Offset)
+    generate_fiu_sigs("txd", "2DRect", Offset)
+    generate_sigs("", "txd", "2DRectShadow", Offset | Single);
     generate_fiu_sigs("txd", "1DArray", Offset)
     generate_fiu_sigs("txd", "2DArray", Offset)
     generate_sigs("", "txd", "1DShadow", Offset | Single, 1);
@@ -328,6 +373,9 @@ def generate_texture_functions(fs):
     generate_fiu_sigs("txd", "2D", Proj)
     generate_fiu_sigs("txd", "2D", Proj, 1)
     generate_fiu_sigs("txd", "3D", Proj)
+    generate_fiu_sigs("txd", "2DRect", Proj)
+    generate_fiu_sigs("txd", "2DRect", Proj, 1)
+    generate_sigs("", "txd", "2DRectShadow", Proj | Single);
     generate_sigs("", "txd", "1DShadow", Proj | Single, 1);
     generate_sigs("", "txd", "2DShadow", Proj | Single);
     end_function(fs, "textureProjGrad")
@@ -338,6 +386,9 @@ def generate_texture_functions(fs):
     generate_fiu_sigs("txd", "2D", Proj | Offset)
     generate_fiu_sigs("txd", "2D", Proj | Offset, 1)
     generate_fiu_sigs("txd", "3D", Proj | Offset)
+    generate_fiu_sigs("txd", "2DRect", Proj | Offset)
+    generate_fiu_sigs("txd", "2DRect", Proj | Offset, 1)
+    generate_sigs("", "txd", "2DRectShadow", Proj | Offset | Single);
     generate_sigs("", "txd", "1DShadow", Proj | Offset | Single, 1);
     generate_sigs("", "txd", "2DShadow", Proj | Offset | Single);
     end_function(fs, "textureProjGradOffset")
