@@ -26,19 +26,8 @@
 #ifndef GLX_HEADER_H
 #define GLX_HEADER_H
 
-#ifdef __VMS
-#include <GL/vms_x_fix.h>
-#endif
+#include "main/glheader.h"
 
-#include "glheader.h"
-
-#ifdef XFree86Server
-
-# include "xorg-server.h"
-# include "resource.h"
-# include "windowstr.h"
-
-#else
 
 # include <X11/Xlib.h>
 # include <X11/Xlibint.h>
@@ -51,7 +40,6 @@
 # include <GL/glx.h>
 # include <sys/time.h>
 
-#endif
 
 
 
