@@ -27,20 +27,36 @@
 #define FEEDBACK_H
 
 
+#include "main/mfeatures.h"
 #include "main/mtypes.h"
 
 
-#if FEATURE_feedback
+void GLAPIENTRY
+_mesa_FeedbackBuffer( GLsizei size, GLenum type, GLfloat *buffer );
+void GLAPIENTRY
+_mesa_PassThrough( GLfloat token );
+void GLAPIENTRY
+_mesa_SelectBuffer( GLsizei size, GLuint *buffer );
+void GLAPIENTRY
+_mesa_InitNames( void );
+void GLAPIENTRY
+_mesa_LoadName( GLuint name );
+void GLAPIENTRY
+_mesa_PushName( GLuint name );
+void GLAPIENTRY
+_mesa_PopName( void );
+GLint GLAPIENTRY
+_mesa_RenderMode( GLenum mode );
 
 extern void
-_mesa_feedback_vertex( GLcontext *ctx,
+_mesa_feedback_vertex( struct gl_context *ctx,
                        const GLfloat win[4],
                        const GLfloat color[4],
                        const GLfloat texcoord[4] );
 
 
-static INLINE void
-_mesa_feedback_token( GLcontext *ctx, GLfloat token )
+static inline void
+_mesa_feedback_token( struct gl_context *ctx, GLfloat token )
 {
    if (ctx->Feedback.Count < ctx->Feedback.BufferSize) {
       ctx->Feedback.Buffer[ctx->Feedback.Count] = token;
@@ -50,49 +66,10 @@ _mesa_feedback_token( GLcontext *ctx, GLfloat token )
 
 
 extern void
-_mesa_update_hitflag( GLcontext *ctx, GLfloat z );
+_mesa_update_hitflag( struct gl_context *ctx, GLfloat z );
 
 
 extern void
-_mesa_init_feedback_dispatch(struct _glapi_table *disp);
-
-#else /* FEATURE_feedback */
-
-#include "main/compiler.h"
-
-static INLINE void
-_mesa_feedback_vertex( GLcontext *ctx,
-                       const GLfloat win[4],
-                       const GLfloat color[4],
-                       const GLfloat texcoord[4] )
-{
-   /* render mode is always GL_RENDER */
-   ASSERT_NO_FEATURE();
-}
-
-
-static INLINE void
-_mesa_feedback_token( GLcontext *ctx, GLfloat token )
-{
-   /* render mode is always GL_RENDER */
-   ASSERT_NO_FEATURE();
-}
-
-static INLINE void
-_mesa_update_hitflag( GLcontext *ctx, GLfloat z )
-{
-   /* render mode is always GL_RENDER */
-   ASSERT_NO_FEATURE();
-}
-
-static INLINE void
-_mesa_init_feedback_dispatch(struct _glapi_table *disp)
-{
-}
-
-#endif /* FEATURE_feedback */
-
-extern void
-_mesa_init_feedback( GLcontext *ctx );
+_mesa_init_feedback( struct gl_context *ctx );
 
 #endif /* FEEDBACK_H */
